<?php

namespace Mobicommerce\GeoFencing\Helper;

use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\Exception\LocalizedException;
use Magento\Store\Model\ScopeInterface;
use Mobicommerce\GeoFencing\Model\ResourceModel\Mapping\CollectionFactory;
use Magento\Store\Model\StoreManagerInterface;

class Data extends AbstractHelper
{
    /**
     * @var CollectionFactory
     */
    private CollectionFactory $collectionFactory;

    /**
     * @var StoreManagerInterface
     */
    private StoreManagerInterface $storeManager;

    /**
     * Constructor
     *
     * @param Context $context
     * @param CollectionFactory $collectionFactory
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(
        Context $context,
        CollectionFactory $collectionFactory,
        StoreManagerInterface $storeManager
    ) {
        parent::__construct($context);
        $this->collectionFactory = $collectionFactory;
        $this->storeManager = $storeManager;
    }

    /**
     * Get Website ID from provided latitude and longitude
     *
     * @param $latitude
     * @param $longitude
     * @param bool $isMultiple
     * @return int|bool|array
     */
    public function getWebsiteIdFromLatLng($latitude, $longitude, bool $isMultiple = false): bool|array|int
    {
        $result = false;

        $collection = $this->collectionFactory->create();
        $collection->getSelect()->where('website_id != ?', 0);

        $latlng = $latitude . ' ' . $longitude;

        $collection->getSelect()
            ->where(new \Zend_Db_Expr("is_exclude = 0"))
            ->where(new \Zend_Db_Expr("poly IS NOT NULL"))
            ->where(new \Zend_Db_Expr("poly != ''"))
            ->where(new \Zend_Db_Expr("MBRContains(ST_GeomFromText(`poly`),ST_GeomFromText('Point($latlng)'))"));

        if ($collection->getSize() > 0) {
            if ($isMultiple) {
                $result = [];
                foreach ($collection as $item) {
                    $result[] = (int) $item->getWebsiteId();
                }

                $result = array_unique($result);
            } else {
                $result = (int) $collection->getFirstItem()->getWebsiteId();
            }
        }

        return $this->applyExcludeFilter($result, $latlng);
    }

    /**
     * Apply to exclude filter with is_exclude to be 1
     *
     * @param int|bool|array $result
     * @param string $latlng
     * @return int|bool|array
     */
    public function applyExcludeFilter(int|bool|array $result, string $latlng): array|bool|int
    {
        if ($result) {
            if (is_array($result)) {
                $filteredResult = [];
                foreach ($result as $websiteId) {
                    if (!$this->isExcludeForWebsite($websiteId, $latlng)) {
                        $filteredResult[] = $websiteId;
                    }
                }

                $result = $filteredResult;
                if (!$result) {
                    $result = false;
                }
            } elseif (is_int($result)) {
                if ($this->isExcludeForWebsite($result, $latlng)) {
                    $result = false;
                }
            }
        }

        return $result;
    }

    /**
     * Check if latlng is not in excluded zone of website
     *
     * @param int $websiteId
     * @param string $latlng
     * @return bool
     */
    public function isExcludeForWebsite(int $websiteId, string $latlng): bool
    {
        $collection = $this->collectionFactory->create();
        $collection->getSelect()->where('website_id = ?', $websiteId);

        $collection->getSelect()
            ->where(new \Zend_Db_Expr("is_exclude = 1"))
            ->where(new \Zend_Db_Expr("poly IS NOT NULL"))
            ->where(new \Zend_Db_Expr("poly != ''"))
            ->where(new \Zend_Db_Expr("MBRContains(ST_GeomFromText(`poly`),ST_GeomFromText('Point($latlng)'))"));

        return $collection->getSize() > 0;
    }

    /**
     * @param int $websiteId
     * @param string $language
     * @return array|null
     * @throws LocalizedException
     */
    public function getDestinationWebsiteStore(int $websiteId, string $language): null|array
    {
        $newStore = null;
        $stores = $this->storeManager->getWebsite($websiteId)->getStores();
        if ($stores) {
            foreach ($stores as $store) {
                $storeLanguage = $this->scopeConfig->getValue(
                    "general/locale/code",
                    ScopeInterface::SCOPE_STORE,
                    $store->getId()
                );

                if ($language == $storeLanguage) {
                    $newStore = $store;
                }
            }

            if (!$newStore) {
                $newStore = $this->storeManager->getWebsite($websiteId)->getDefaultStore();
            }
        }

        if (!$newStore) {
            return null;
        }

        return [
            'website_code' => $this->storeManager->getWebsite($websiteId)->getCode(),
            'store_code' => $newStore->getCode(),
            'store_id' => $newStore->getId()
        ];
    }

    /**
     * @param string $language
     * @return array
     */
    public function getAllWebsites(string $language): array
    {
        $result = [];
        foreach ($this->storeManager->getWebsites() as $website) {
            $item = $this->getDestinationWebsiteStore($website->getId(), $language);
            if ($item) {
                $result[] = $item;
            }
        }

        return $result;
    }
}
