define([
    'jquery',
    'underscore',
    'uiRegistry',
    'Magento_PageBuilder/js/form/element/map',
    'Magento_PageBuilder/js/utils/map',
    'Magento_PageBuilder/js/events',
    'uiRegistry',
    'Magento_Ui/js/modal/confirm'
], function ($, _, registry, Element, GoogleMap, events, uiRegistry, confirmation) {
    'use strict';

    let google = window.google || {};
    let poly = null;
    let mc_gf_defaults = {
        latitude: 30.2672,
        longitude: -97.7431,
        zoom: 8
    };

    return Element.extend({
        renderMap: function(element) {
            let self = this;

            if (!this.apiKeyValid()) {
                return;
            }

            let latitude = uiRegistry.get('index = position[latitude]').value();
            let longitude = uiRegistry.get('index = position[longitude]').value();
            let zoom = uiRegistry.get('index = zoom').value();

            if (latitude.length == 0) {
                latitude = mc_gf_defaults.latitude;
            }

            if (longitude.length == 0) {
                longitude = mc_gf_defaults.longitude;
            }

            if (zoom.length == 0) {
                zoom = mc_gf_defaults.zoom;
            }

            let mapOptions = {
                center: { lat: parseFloat(latitude), lng: parseFloat(longitude) },
                zoom: parseInt(zoom),
                navigationControl: true,
                mapTypeId: google.maps.MapTypeId.ROADMAP,
                mapTypeControl: false,
                draggable: true
            };

            events.on('googleMaps:authFailure', function () {
                this.apiKeyValid(false);
            }.bind(this));

            // Create the map
            this.mapElement = new GoogleMap(element, [], mapOptions);

            if (!this.mapElement || !this.mapElement.map) {
                return;
            }

            self.initMapChangeEvents();
            self.initAutoCompleteElement();
            self.initPoly();
            self.initClearButtonClick();
        },

        onUpdate: function() {

        },

        initMapChangeEvents: function() {
            let self = this;

            google.maps.event.addListener(self.mapElement.map, 'zoom_changed', function() {
                self.updatePath();
            });

            google.maps.event.addListener(self.mapElement.map, 'center_changed', function() {
                self.updatePath();
            });
        },

        initPoly: function() {
            let self = this;
            poly = new google.maps.Polygon({
                strokeColor: "#FF0000",
                strokeOpacity: 0.8,
                strokeWeight: 3,
                fillColor: "#FF0000",
                fillOpacity: 0.35,
                draggable: true,
                editable: true
            });

            poly.setMap(self.mapElement.map);
            self.fillPolyPathFromDb();
            google.maps.event.addListener(this.mapElement.map, "click", (event) => {
                const path = poly.getPath();
                path.push(event.latLng);
                self.updatePath();
            });

            google.maps.event.addListener(poly.getPath(), 'insert_at', function(index, obj) {
                self.updatePath();
            });

            google.maps.event.addListener(poly.getPath(), 'set_at', function(index, obj) {
                self.updatePath();
            });

            google.maps.event.addListener(poly, 'dragend', function (event) {
                self.updatePath();
            });

            google.maps.event.addListener(poly, 'bounds_changed', function (event) {
                self.updatePath();
            });
        },

        initAutoCompleteElement: function() {
            let self = this;
            let element = document.getElementsByName('location_search')[0];
            let autocomplete = new google.maps.places.Autocomplete(element);
            autocomplete.setFields([
                'address_components',
                'geometry',
                'icon',
                'name'
            ]);

            autocomplete.addListener('place_changed', function () {
                let place = autocomplete.getPlace();
                if (!place.geometry) {
                    window.alert("No details available for input: '" + place.name + "'");
                    return;
                }

                if (place.geometry.viewport) {
                    self.mapElement.map.fitBounds(place.geometry.viewport);
                } else {
                    self.mapElement.map.setCenter(place.geometry.location);
                }
            });
        },

        updatePath: function() {
            let self = this;

            const mapCenter = self.mapElement.map.getCenter();
            const mapZoom = self.mapElement.map.getZoom();

            uiRegistry.get('index = position[latitude]').value(mapCenter.lat());
            uiRegistry.get('index = position[longitude]').value(mapCenter.lng());
            uiRegistry.get('index = zoom').value(mapZoom);

            const vertices = poly.getPath();
            const encodeString = google.maps.geometry.encoding.encodePath(vertices);
            if (encodeString) {
                uiRegistry.get('index = encoded').value(encodeString);
                self.updatePolyCordinate();
            }
        },

        updatePolyCordinate: function() {
            const vertices = poly.getPath();
            let contentString="Polygon((";
            for (let i = 0; i < vertices.getLength(); i++) {
                const xy = vertices.getAt(i);
                contentString += xy.lat() + " " + xy.lng() +  ",";
            }
            const firstele = vertices.getAt(0);
            contentString += firstele.lat() + " " + firstele.lng() + "))";

            uiRegistry.get('index = poly').value(contentString);
        },

        fillPolyPathFromDb: function() {
            const encodeString = uiRegistry.get('index = encoded').value();
            const vertices = google.maps.geometry.encoding.decodePath(encodeString);

            poly.setPath(vertices);
        },

        clearPath: function() {
            confirmation({
                title: $.mage.__('Confirmation'),
                content: $.mage.__('Are you sure want to clear the GEO Fencing ?'),
                actions: {
                    confirm: function(){
                        poly.setPath([]);
                        uiRegistry.get('index = encoded').value('');
                        uiRegistry.get('index = poly').value('');
                    },
                    cancel: function(){},
                    always: function(){}
                }
            });
        },

        initClearButtonClick: function() {
            let self = this;

            if ($("button[data-index='clear_poly_button']").length > 0) {
                $("button[data-index='clear_poly_button']").on( "click", function() {
                    self.clearPath();
                });
            } else {
                setTimeout(function() {
                    self.initClearButtonClick();
                }, 1000);
            }

        }
    });
});
