<?php
/**
 * Mobicommerce
 *
 * Copyright (C) 2021 Mobicommerce <info@mobicommerce.net>
 *
 * @category Mobicommerce
 * @package Mobicommerce_Mobiapp
 * @copyright Copyright (c) 2021 Mobicommerce (http://www.mobicommerce.net/)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License,version 3 (GPL-3.0)
 * @author Mobicommerce <info@mobicommerce.net>
 */

namespace Mobicommerce\Mobiapp\Helper;

use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\Module\Dir\Reader;
use Magento\Framework\Xml\Parser;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Store\Model\ScopeInterface;

class Translation extends AbstractHelper
{
    public const DEFAULT_LOCALE = 'en_US';
    public const CHILD_TRANSLATIONS_FILEPATH = 'mobicommerce/translations';
    public const CUSTOMIZATION_FILENAME = 'customization.xml';

    /**
     * @param Context $context
     * @param Reader $reader
     * @param Parser $parser
     * @param Filesystem $filesystem
     * @param File $filesystemIo
     */
    public function __construct(
        Context $context,
        Reader $reader,
        Parser $parser,
        \Magento\Framework\Filesystem $filesystem,
        \Magento\Framework\Filesystem\Io\File $filesystemIo
    ) {
        parent::__construct($context);
        $this->reader = $reader;
        $this->parser = $parser;
        $this->filesystem = $filesystem;
        $this->filesystemIo = $filesystemIo;
    }

    /**
     * GetTranslationsByLocale function
     *
     * @param String $locale
     */
    public function getTranslationsByLocale($locale)
    {
        $moduleDir = $this->reader->getModuleDir('etc', 'Mobicommerce_Mobiapp');
        $localeFile =  $moduleDir. '/translations/'.$locale.'.xml';
        if (!$this->filesystemIo->fileExists($localeFile)) {
            $localeFile =  $moduleDir. '/translations/'.self::DEFAULT_LOCALE.'.xml';
        }

        $items = $this->parser->load($localeFile)->xmlToArray();
        $items = $this->addCustomizedTranslations($items);

        $result = $items['config']['items'];
        $this->addCodeKey($result);
        $result = $this->mergeSavedTranslationsByLocale($result, $locale);
        $this->addCodeKey($result);
        return $result;
    }

    /**
     * GetTranslationsByCurrentStore function
     */
    public function getTranslationsByCurrentStore()
    {
        $locale = $this->scopeConfig->getValue('general/locale/code', ScopeInterface::SCOPE_STORE);

        return $this->getTranslationsByLocale($locale);
    }

    /**
     * AddCustomizedTranslations function
     *
     * @param Array $items
     */
    public function addCustomizedTranslations($items)
    {
        $varDirectory = $this->getVarDirectory();
        $localeFile =  $varDirectory . self::CHILD_TRANSLATIONS_FILEPATH . '/' . self::CUSTOMIZATION_FILENAME;

        if ($this->filesystemIo->fileExists($localeFile)) {
            $customizedTranslations = $this->parser->load($localeFile)->xmlToArray();

            $items = array_merge_recursive($items, $customizedTranslations);
        }

        return $items;
    }

    /**
     * AddCodeKey function
     *
     * @param Array $result
     */
    public function addCodeKey(&$result)
    {
        foreach ($result as $key => $item) {
            if (!isset($item['type'])) {
                if ($item[0]['type']) {
                    $result[$key] = $item[0];
                } else {
                    continue;
                }
            }
            $result[$key]['code'] = $key;
        }
    }

    /**
     * MergeSavedTranslationsByLocale function
     *
     * @param Array $result
     * @param String $locale
     */
    private function mergeSavedTranslationsByLocale($result, $locale)
    {
        $varDirectory = $this->getVarDirectory();
        $localeFile =  $varDirectory. self::CHILD_TRANSLATIONS_FILEPATH . '/' . $locale . '.xml';

        if ($this->filesystemIo->fileExists($localeFile)) {
            $items = $this->parser->load($localeFile)->xmlToArray();
            $saved = $items['config']['items'];

            foreach ($saved as $code => $item) {
                if (array_key_exists($code, $result)) {
                    $result[$code]['code'] = $code;
                    $result[$code]['value'] = $item['value'];
                }
            }
        }
        
        return $result;
    }

    /**
     * SaveTranslationsByLocale function
     *
     * @param String $locale
     * @param Array $data
     */
    public function saveTranslationsByLocale($locale, $data)
    {
        $varDirectory = $this->getVarDirectory();
        $this->filesystemIo->checkAndCreateFolder($varDirectory . self::CHILD_TRANSLATIONS_FILEPATH);
        $localeFile =  $varDirectory . self::CHILD_TRANSLATIONS_FILEPATH . '/' . $locale . '.xml';
        if ($this->filesystemIo->fileExists($localeFile)) {
            $this->filesystemIo->rm($localeFile);
        }

        $translations = [];
        foreach (["labels", "messages"] as $key) {
            foreach ($data[$key] as $code => $value) {
                $translations[$code] = $value;
            }
        }

        $xmlRoot = '';
        $xmlRoot .= '<?xml version="1.0"?>';
        $xmlRoot .= '<config xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"></config>';
        $xml = new \SimpleXMLElement($xmlRoot);
        $itemsNode = $xml->addChild('items');

        foreach ($translations as $code => $value) {
            $itemNode = $itemsNode->addChild($code);
           // $itemNode->addChild('value', $value);
           $itemNode->addChild('value', htmlspecialchars($value));

        }
        
        $xml->asXML($localeFile);
    }

    /**
     * GetVarDirectory function
     */
    protected function getVarDirectory()
    {
        return $this->filesystem->getDirectoryRead(DirectoryList::VAR_DIR)->getAbsolutePath();
    }
}
