<?php
/**
 * Mobicommerce
 * Copyright (C) 2021 Mobicommerce <info@mobicommerce.net>
 * @category Mobicommerce
 * @package Mobicommerce_Mobiapp
 * @copyright Copyright (c) 2021 Mobicommerce (http://www.mobicommerce.net/)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License,version 3 (GPL-3.0)
 * @author Mobicommerce <info@mobicommerce.net>
 */

namespace Mobicommerce\Mobiapp\Model;

use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\App\ResourceConnection;
use Mobicommerce\Mobiapp\Model\ResourceModel\Customsegment\CollectionFactory as CustomsegmentCollectionFactory;
use Mobicommerce\Mobiapp\Model\ResourceModel\Customsegmentlog\CollectionFactory as CustomsegmentlogCollectionFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Serialize\Serializer\Json;

class CustomsegmentRepository
{
    /**
     * @var CustomsegmentFactory
     */
    private CustomsegmentFactory $customsegmentFactory;

    /**
     * @var ResourceConnection
     */
    private ResourceConnection $resourceConnection;

    /**
     * @var CustomsegmentCollectionFactory
     */
    private CustomsegmentCollectionFactory $customsegmentCollectionFactory;

    /**
     * @var ScopeConfigInterface
     */
    private ScopeConfigInterface $scopeConfig;

    /**
     * @var Json
     */
    private Json $json;

    /**
     * Constructor
     *
     * @param CustomsegmentFactory $customsegmentFactory
     * @param ResourceConnection $resourceConnection
     * @param CustomsegmentCollectionFactory $customsegmentCollectionFactory
     * @param CustomsegmentlogCollectionFactory $customsegmentlogCollectionFactory
     * @param ScopeConfigInterface $scopeConfig
     * @param Json $json
     */
    public function __construct(
        CustomsegmentFactory $customsegmentFactory,
        ResourceConnection $resourceConnection,
        CustomsegmentCollectionFactory $customsegmentCollectionFactory,
        CustomsegmentlogCollectionFactory $customsegmentlogCollectionFactory,
        ScopeConfigInterface $scopeConfig,
        Json $json
    ) {
        $this->customsegmentFactory = $customsegmentFactory;
        $this->resourceConnection = $resourceConnection;
        $this->customsegmentCollectionFactory = $customsegmentCollectionFactory;
        $this->customsegmentlogCollectionFactory = $customsegmentlogCollectionFactory;
        $this->scopeConfig = $scopeConfig;
        $this->json = $json;
    }

    /**
     * Add queue for add
     *
     * @param int $id
     * @return void
     * @throws LocalizedException
     */
    public function queueAdd(int $id)
    {
        $model = $this->customsegmentFactory->create();
        $model->load($id);

        if (!$model->getId()) {
            throw new LocalizedException(__('Item doesnt exist'));
        }

        if ($model->getStatus() != 0) {
            throw new LocalizedException(__('Item not in pending state'));
        }

        $model->setStatus(1)->save();

        $emails = $this->getIndividualEmails($model);
        $insertArray = [];
        foreach ($emails as $email) {
            $insertArray[] = [
                'segment_id' => $model->getId(),
                'email' => $email,
                'action' => 'add',
                'status' => 0
            ];
        }

        $table = $this->resourceConnection->getTableName('mobicommerce_custom_segment_log');
        return $this->resourceConnection->getConnection()->insertMultiple($table, $insertArray);
    }

    /**
     * Add queue for soft delete
     *
     * @param int $id
     * @return void
     * @throws LocalizedException
     */
    public function queueSoftDelete(int $id)
    {
        $model = $this->customsegmentFactory->create();
        $model->load($id);

        if (!$model->getId()) {
            throw new LocalizedException(__('Item doesnt exist'));
        }

        if ($model->getStatus() != 2) {
            throw new LocalizedException(__('Item not in queued add state'));
        }

        $model->setStatus(3)->save();

        $emails = $this->getIndividualEmails($model);
        $insertArray = [];
        foreach ($emails as $email) {
            $insertArray[] = [
                'segment_id' => $model->getId(),
                'email' => $email,
                'action' => 'delete',
                'status' => 0
            ];
        }

        $table = $this->resourceConnection->getTableName('mobicommerce_custom_segment_log');
        return $this->resourceConnection->getConnection()->insertMultiple($table, $insertArray);
    }

    /**
     * To hard delete
     *
     * @param int $id
     * @return void
     * @throws LocalizedException
     */
    public function delete(int $id): void
    {
        $model = $this->customsegmentFactory->create();
        $model->load($id);

        if (!$model->getId()) {
            throw new LocalizedException(__('Item doesnt exist'));
        }

        if (!in_array($model->getStatus(), [0,4])) {
            throw new LocalizedException(__('This item cannot be deleted as it is in intermediate stage.'));
        }

        $model->delete();
    }

    /**
     * Get Individual emails separated by comma
     *
     * @param Customsegment $model
     * @return array
     */
    public function getIndividualEmails(Customsegment $model)
    {
        $emails = $model->getEmails();
        $emails = explode(',', $emails);
        $emails = array_unique(array_filter(array_map('trim', $emails)));
        return $emails;
    }

    /**
     * Get count of Individual emails separated by comma
     *
     * @param Customsegment $model
     * @return int
     */
    public function getIndividualEmailsCount(Customsegment $model): int
    {
        return count($this->getIndividualEmails($model));
    }

    /**
     * Process the queue
     *
     * @return void
     */
    public function processQueue(): void
    {
        $segmentCollection = $this->customsegmentCollectionFactory->create();
        $segmentCollection->addFieldToFilter('status', [
            'in' => [1,3]
        ]);
        $segmentCollection->getSelect()->order('id', 'ASC');
        $segmentCollection->getSelect()->limit(1);

        if ($segmentCollection->getSize()) {
            foreach ($segmentCollection as $collection) {
                $segmentLogCollection = $this->customsegmentlogCollectionFactory->create();
                $segmentLogCollection->getSelect()->order('id', 'ASC');
                $segmentLogCollection
                    ->addFieldToFilter('segment_id', $collection->getId())
                    ->addFieldToFilter('status', 0);

                if ($segmentLogCollection->getSize()) {
                    foreach ($segmentLogCollection as $item) {
                        $item->setStatus(3)
                            ->setExecutedAt(date('Y-m-d H:i:s'))
                            ->save();
                        try {
                            $this->syncCustomSegmentWithOneSignal($item);
                            $item->setStatus(1);
                        } catch (\Exception $e) {
                            $item->setStatus(2)
                                ->setErrorMessage($e->getMessage());
                        }

                        $item->save();
                    }
                }

                switch ($collection->getStatus()) {
                    case 1:
                        $collection->setStatus(2);
                        break;
                    case 3:
                        $collection->setStatus(4);
                }

                $collection->save();
            }
        }
    }

    /**
     * Sync with actual onesignal service
     *
     * @param Customsegmentlog $model
     * @return void
     * @throws LocalizedException
     */
    public function syncCustomSegmentWithOneSignal(Customsegmentlog $model): void
    {
        $appId = $this->scopeConfig->getValue(
            'mobi_notification/onesignalnotification/app_id'
        );
        $email = $model->getEmail();
        $token = $this->scopeConfig->getValue(
            'mobi_notification/onesignalnotification/rest_api_key'
        );

        if (!$appId || !$token) {
            throw new LocalizedException(__('App ID or Token not defined in store configuration'));
        }

        $url = "https://onesignal.com/api/v1/apps/$appId/users/$email";

        $key = 'segment_' . $model->getSegmentId();
        $value = ($model->getAction() == 'add') ? '1' : '';
        $params = $this->json->serialize([
            'tags' => [
                $key => $value
            ]
        ]);

        $curl = curl_init();

        curl_setopt_array($curl, array(
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 10000,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'PUT',
            CURLOPT_POSTFIELDS => $params,
            CURLOPT_HTTPHEADER => array(
                "Authorization: Basic $token",
                'Content-Type: application/json'
            ),
        ));

        $response = curl_exec($curl);
        $responseStatusCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        curl_close($curl);
        $response = $this->json->unserialize($response);

        if (isset($response['success']) && $response['success']) {
            return;
        }

        $error = "Invalid response from Onesignal side";
        if (isset($response['errors']) && $response['errors']) {
            $error = implode(', ', $response['errors']);
        }

        throw new LocalizedException(__($error));
    }
}
