<?php
/**
 * Mobicommerce
 * Copyright (C) 2021 Mobicommerce <info@mobicommerce.net>
 * @category Mobicommerce
 * @package Mobicommerce_Mobiapp
 * @copyright Copyright (c) 2021 Mobicommerce (http://www.mobicommerce.net/)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License,version 3 (GPL-3.0)
 * @author Mobicommerce <info@mobicommerce.net>
 */

namespace Mobicommerce\Mobiapp\Model;

use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Exception\NotFoundException;
use Magento\Framework\Event\ManagerInterface as EventManager;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Store\Model\ScopeInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Cms\Model\PageFactory;

class PageBuilderRepository
{
    public const TBL_CATEGORY_MAPPING = 'mobicommerce_page_builder_category_mapping';
    public const TBL_LINK_BUILDER_MAPPING = 'mobicommerce_page_builder_link_builder_widget_mapping';
    public const TBL_SECTION_MAPPING ='mobicommerce_page_builder_section_mapping';

    /**
     * PageBuilderRepository constructor.
     *
     * @param \Mobicommerce\Mobiapp\Model\ResourceModel\PageBuilder $resource
     * @param \Mobicommerce\Mobiapp\Model\PageBuilderFactory $pageBuilderFactory
     * @param Magento\Framework\App\ResourceConnection $resourceConnection
     * @param Magento\Framework\Event\ManagerInterface $eventManager
     * @param Magento\Framework\Serialize\Serializer\Json $json
     * @param Magento\Store\Model\ScopeInterface $scopeConfig
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param Magento\Cms\Model\PageFactory $cmsPageFactory
     */
    public function __construct(
        \Mobicommerce\Mobiapp\Model\ResourceModel\PageBuilder $resource,
        \Mobicommerce\Mobiapp\Model\PageBuilderFactory $pageBuilderFactory,
        ResourceConnection $resourceConnection,
        EventManager $eventManager,
        Json $json,
        ScopeConfigInterface $scopeConfig,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        PageFactory $cmsPageFactory
    ) {
        $this->resource = $resource;
        $this->pageBuilderFactory = $pageBuilderFactory;
        $this->resourceConnection = $resourceConnection;
        $this->eventManager = $eventManager;
        $this->json = $json;
        $this->scopeConfig = $scopeConfig;
        $this->storeManager = $storeManager;
        $this->cmsPageFactory = $cmsPageFactory;
    }

    /**
     * Save
     *
     * @param object $pageBuilder
     */
    public function save($pageBuilder)
    {
        $this->resource->save($pageBuilder);
        $this->saveCategoriesForPageBuilder($pageBuilder);
        $this->saveLinkBuilderWidgets($pageBuilder);
        $this->saveSections($pageBuilder);
    }

    /**
     * Save categories for page builder
     *
     * @param object $pageBuilder
     */
    public function saveCategoriesForPageBuilder($pageBuilder)
    {
        if ($categoryIds = $pageBuilder->getData('category_ids')) {
            if ($categoryIds) {
                $table = $this->resourceConnection->getTableName(self::TBL_CATEGORY_MAPPING);
                $connection = $this->resourceConnection->getConnection();
                $select = $connection->select()
                    ->from(
                        ['mapping' => $table]
                    )->where('page_builder_id = ?', $pageBuilder->getId())
                    ->reset(\Magento\Framework\DB\Select::COLUMNS)
                    ->columns(['GROUP_CONCAT(category_id) AS db_category_ids']);
                $dbCategoryIds = $connection->fetchOne($select);
                if ($dbCategoryIds) {
                    $dbCategoryIds = explode(',', $dbCategoryIds);
                }

                if (!$dbCategoryIds) {
                    $dbCategoryIds = [];
                }

                $categoryIdsToInsert = array_diff($categoryIds, $dbCategoryIds);
                if ($categoryIdsToInsert) {
                    $bulkInsert = [];
                    foreach ($categoryIdsToInsert as $categoryId) {
                        $bulkInsert[] = [
                            'page_builder_id' => $pageBuilder->getId(),
                            'category_id' => $categoryId
                        ];
                    }

                    $connection->insertMultiple($table, $bulkInsert);
                }

                $categoryIdsToDelete = array_diff($dbCategoryIds, $categoryIds);
                if ($categoryIdsToDelete) {
                    $connection->delete(
                        $table,
                        [
                            'page_builder_id = ?' => $pageBuilder->getId(),
                            'category_id IN (?)' => $categoryIdsToDelete
                        ]
                    );
                }
            }

            $pageBuilder->unsetData('category_ids');
        }

        if ($categoryPosition = $pageBuilder->getData('pagebuilder_category_position')) {
            $categoryPosition = $this->json->unserialize($categoryPosition);
            if ($categoryPosition) {
                $table = $this->resourceConnection->getTableName(self::TBL_CATEGORY_MAPPING);
                $connection = $this->resourceConnection->getConnection();

                foreach ($categoryPosition as $categoryId => $position) {
                    $connection->update($table, [
                        'position' => $position
                    ], [
                        'page_builder_id = ?' => $pageBuilder->getId(),
                        'category_id = ?' => $categoryId
                    ]);
                }
            }

            $pageBuilder->unsetData('pagebuilder_category_position');
        }
    }

    /**
     * Save link widgets
     *
     * @param object $pageBuilder
     */
    public function saveLinkBuilderWidgets($pageBuilder)
    {
        $connection = $this->resourceConnection->getConnection();
        $table = $this->resourceConnection->getTableName(self::TBL_LINK_BUILDER_MAPPING);

        $dynamicRowsField = 'link_builder_dynamic_rows';
        if ($linkBuilderWidgets = $pageBuilder->getData($dynamicRowsField)) {
            $linkBuilderWidgets = $linkBuilderWidgets[$dynamicRowsField] ?? false;

            if ($linkBuilderWidgets) {
                $allWidgetIds = [];
                foreach ($linkBuilderWidgets as $row) {
                    $widgetId = $row['id'];
                    $position = $row['position'];

                    $allWidgetIds[] = $widgetId;

                    $select = $connection->select()
                        ->from(
                            ['mapping' => $table]
                        )->where('page_builder_id = ?', $pageBuilder->getId())
                        ->where('link_builder_widget_id = ?', $widgetId);

                    if ($record = $connection->fetchRow($select)) {
                        if ($record['position'] != $position) {
                            $connection->update($table, [
                                'position' => $position
                            ], [
                                'page_builder_id = ?' => $pageBuilder->getId(),
                                'link_builder_widget_id = ?' => $widgetId
                            ]);
                        }
                        
                    } else {
                        $connection->insert($table, [
                            'page_builder_id' => $pageBuilder->getId(),
                            'link_builder_widget_id' => $widgetId,
                            'position' => $position
                        ]);
                    }
                }
                
                $connection->delete($table, [
                    'page_builder_id = ?' => $pageBuilder->getId(),
                    'link_builder_widget_id NOT IN (?)' => $allWidgetIds
                ]);
            }

            $pageBuilder->unsetData($dynamicRowsField);
        } else {
            $connection->delete($table, [
                "page_builder_id = ?" => $pageBuilder->getId()
            ]);
        }
    }

    /**
     * Save sections
     *
     * @param object $pageBuilder
     */
    public function saveSections($pageBuilder)
    {
        $connection = $this->resourceConnection->getConnection();
        $table = $this->resourceConnection->getTableName(self::TBL_SECTION_MAPPING);

        $dynamicRowsField = 'sections_dynamic_rows';
        if ($sections = $pageBuilder->getData($dynamicRowsField)) {
            $sections = $sections[$dynamicRowsField] ?? false;

            if ($sections) {
                $allSectionIds = [];
                foreach ($sections as $row) {
                    $sectionId = $row['id'];
                    $position = $row['position'];

                    $allSectionIds[] = $sectionId;

                    $select = $connection->select()
                        ->from(
                            ['mapping' => $table]
                        )->where('page_builder_id = ?', $pageBuilder->getId())
                        ->where('section_id = ?', $sectionId);

                    if ($record = $connection->fetchRow($select)) {
                        if ($record['position'] != $position) {
                            $connection->update($table, [
                                'position' => $position
                            ], [
                                'page_builder_id = ?' => $pageBuilder->getId(),
                                'section_id = ?' => $sectionId
                            ]);
                        }
                        
                    } else {
                        $connection->insert($table, [
                            'page_builder_id' => $pageBuilder->getId(),
                            'section_id' => $sectionId,
                            'position' => $position
                        ]);
                    }
                }
                
                $connection->delete($table, [
                    'page_builder_id = ?' => $pageBuilder->getId(),
                    'section_id NOT IN (?)' => $allSectionIds
                ]);
            }

            $pageBuilder->unsetData($dynamicRowsField);
        } else {
            $connection->delete($table, [
                "page_builder_id = ?" => $pageBuilder->getId()
            ]);
        }
    }

    /**
     * Check duplicate ID
     *
     * @param int $id
     * @return object
     */
    public function duplicateById($id)
    {
        $model = $this->pageBuilderFactory->create()->load($id);

        if (!$model->getId()) {
            throw new NotFoundException(__('Item not available to duplicate'));
        }

        $model->setId(null)
            ->setName('Copy of '. $model->getName())
            ->unsetData('created_at')
            ->unsetData('updated_at')
            ->save();

        $this->eventManager->dispatch('mobicommerce_pagebuilder_duplicate_child_process', [
            'id' => $id,
            'pagebuilder' => $model
        ]);

        return $model;
    }

    /**
     * Get by id
     *
     * @param int $id
     * @return object
     */
    public function getById($id)
    {
        $model = $this->pageBuilderFactory->create()->load($id);

        if (!$model->getId()) {
            throw new NotFoundException(__('Item not available'));
        }

        return $model;
    }

    /**
     * Get left panle category ids
     *
     * @param int $pageBuilderId
     * @return object
     */
    public function getLeftPanelCategoryIds($pageBuilderId)
    {
        $categoryIds = [];

        $connection = $this->resourceConnection->getConnection();
        $table = $this->resourceConnection->getTableName(self::TBL_CATEGORY_MAPPING);
        $select = $connection->select()
            ->from(
                ['mapping' => $table]
            )->where('page_builder_id = ?', $pageBuilderId)
            ->order('position', 'ASC');
        $collection = $connection->fetchAll($select);
        if ($collection) {
            foreach ($collection as $item) {
                $categoryIds[] = (int) $item['category_id'];
            }
        }

        return $categoryIds;
    }

    /**
     * Get id from homepageconfiguration
     *
     * @param object $type
     * @return int
     */
    public function getIdFromHomepageConfiguration($type)
    {
        $pageBuilderId = null;
        $storeId = $this->storeManager->getStore()->getId();
        $cmsPageId = $this->scopeConfig->getValue(
            'mobi_basic/general/cms_home_page',
            ScopeInterface::SCOPE_STORE,
            $storeId
        );

        if ($cmsPageId) {
            $page = $this->cmsPageFactory->create()->load($cmsPageId);
            if ($page->getId()) {
                switch ($type) {
                    case 'pwa':
                        $pageBuilderId = $page->getMobicommercePwaLandingPageId();
                        break;

                    case 'desktop':
                        $pageBuilderId = $page->getMobicommerceDesktopLandingPageId();
                        break;
                    
                    case 'mobileapp':
                        $pageBuilderId = $page->getMobicommerceMobileappLandingPageId();
                        break;

                    default:
                        // code...
                        break;
                }
            }
        }

        return $pageBuilderId;
    }

     /**
      * Get id from cmsid
      *
      * @param object $type
      * @return int
      */
    public function getIdFromCmsId($type)
    {
        $pageBuilderId = null;
        $storeId = $this->storeManager->getStore()->getId();
        $cmsPageId = $this->scopeConfig->getValue(
            'mobi_multiwebsite/config/store_cms_page',
            ScopeInterface::SCOPE_STORE,
            $storeId
        );

        if ($cmsPageId) {
            $page = $this->cmsPageFactory->create()->load($cmsPageId);
            if ($page->getId()) {
                switch ($type) {
                    case 'pwa':
                        $pageBuilderId = $page->getMobicommercePwaLandingPageId();
                        break;

                    case 'desktop':
                        $pageBuilderId = $page->getMobicommerceDesktopLandingPageId();
                        break;
                    
                    case 'mobileapp':
                        $pageBuilderId = $page->getMobicommerceMobileappLandingPageId();
                        break;

                    default:
                        // code...
                        break;
                }
            }
        }

        return $pageBuilderId;
    }
}
