<?php
/**
 * Mobicommerce
 * Copyright (C) 2021 Mobicommerce <info@mobicommerce.net>
 * @category Mobicommerce
 * @package Mobicommerce_Mobiapp
 * @copyright Copyright (c) 2021 Mobicommerce (http://www.mobicommerce.net/)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License,version 3 (GPL-3.0)
 * @author Mobicommerce <info@mobicommerce.net>
 */

namespace Mobicommerce\Mobiapp\Model;

use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Exception\NotFoundException;
use Magento\Framework\Event\ManagerInterface as EventManager;
use Magento\Store\Model\StoreManagerInterface;

class ProductcollectionRepository
{
    /**
     * ProductcollectionRepository constructor.
     *
     * @param \Mobicommerce\Mobiapp\Model\ResourceModel\Productcollection $resource
     * @param \Mobicommerce\Mobiapp\Model\ProductcollectionFactory $productcollectionFactory
     * @param Magento\Framework\App\ResourceConnection $resourceConnection
     * @param Magento\Framework\Event\ManagerInterface $eventManager
     * @param Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\CatalogRule\Model\RuleFactory $ruleFactory
     * @param \Magento\Cms\Model\PageFactory $cmsPageModelFactory
     */
    public function __construct(
        \Mobicommerce\Mobiapp\Model\ResourceModel\Productcollection $resource,
        \Mobicommerce\Mobiapp\Model\ProductcollectionFactory $productcollectionFactory,
        ResourceConnection $resourceConnection,
        EventManager $eventManager,
        StoreManagerInterface $storeManager,
        \Magento\CatalogRule\Model\RuleFactory $ruleFactory,
        \Magento\Cms\Model\PageFactory $cmsPageModelFactory
    ) {
        $this->resource = $resource;
        $this->productcollectionFactory = $productcollectionFactory;
        $this->resourceConnection = $resourceConnection;
        $this->eventManager = $eventManager;
        $this->storeManager = $storeManager;
        $this->ruleFactory = $ruleFactory;
        $this->cmsPageModelFactory = $cmsPageModelFactory;
    }

    /**
     * Duplicatebyid
     *
     * @param int $id
     * @return $object
     */
    public function duplicateById($id)
    {
        $model = $this->productcollectionFactory->create()->load($id);

        if (!$model->getId()) {
            throw new NotFoundException(__('Item not available to duplicate'));
        }

        $model->setId(null)
            ->setName('Copy of '. $model->getName())
            ->unsetData('created_at')
            ->unsetData('updated_at')
            ->save();

        $this->eventManager->dispatch('mobicommerce_productcollection_duplicate_child_process', [
            'id' => $id,
            'productcollection' => $model
        ]);

        return $model;
    }

    /**
     * Get matched product sku
     *
     * @param array $conditions
     * @return $array
     */
    public function getMatchedProductSkus($conditions)
    {
        $productIds = $this->getMatchedProductIds($conditions);

        $skus = [];
        $connection = $this->resourceConnection->getConnection();
        $table = $this->resourceConnection->getTableName("catalog_product_entity");
        $select = $connection->select()
            ->from(
                ['product' => $table]
            )->where('entity_id IN (?)', $productIds);
        $result = $connection->fetchAll($select);
        if ($result) {
            foreach ($result as $item) {
                $skus[] = $item['sku'];
            }
        }

        return array_unique($skus);
    }

    /**
     * Get matched product skus by cms page id
     *
     * @param int $pageId
     * @return $array
     */
    public function getMatchedProductSkusByCmsPageId($pageId)
    {
        $page = $this->cmsPageModelFactory->create()->load($pageId);

        if (!$page->getId()) {
            return [];
        }

        $productCollectionId = $page->getMobicommerceProductCollectionId();

        return $this->getMatchedProductSkusByProductCollectionId($productCollectionId);
    }

    /**
     * Get matched product skus by product collection id
     *
     * @param int $id
     * @return $array
     */
    public function getMatchedProductSkusByProductCollectionId($id)
    {
        $productcollectionModel = $this->productcollectionFactory->create()->load($id);

        if (!$productcollectionModel->getId()) {
            return [];
        }
        
        $conditions = $productcollectionModel->getBackendConditions();
        $conditions = json_decode($conditions, true);
        return $this->getMatchedProductSkus($conditions);
    }

    /**
     * Get matched product id
     *
     * @param array $conditions
     * @return $array
     */
    public function getMatchedProductIds($conditions)
    {
        /** @var Rule $ruleCondition */
        $ruleCondition = $this->ruleFactory->create();

        $websiteIds = [];
        foreach ($this->storeManager->getWebsites() as $website) {
            $websiteIds[] = $website->getId();
        }

        $ruleCondition->setWebsiteIds($websiteIds);
        $ruleCondition->loadPost($conditions['rule'] ?? []);

        $productIds = array_keys($ruleCondition->getMatchingProductIds());

        return $this->addParentProductIds($productIds);
    }

    /**
     * Add parent product ids
     *
     * @param array $productIds
     * @return $array
     */
    private function addParentProductIds($productIds)
    {
        $connection = $this->resourceConnection->getConnection();
        $table = $this->resourceConnection->getTableName("catalog_product_super_link");
        $select = $connection->select()
            ->from(
                ['link_table' => $table]
            )->where('product_id IN (?)', $productIds)
            ->group('parent_id');
        $result = $connection->fetchAll($select);
        if ($result) {
            foreach ($result as $item) {
                $productIds[] = $item['parent_id'];
            }
        }

        return array_unique($productIds);
    }
}
