<?php
/**
 * Mobicommerce
 * Copyright (C) 2021 Mobicommerce <info@mobicommerce.net>
 * @category Mobicommerce
 * @package Mobicommerce_Mobiapp
 * @copyright Copyright (c) 2021 Mobicommerce (http://www.mobicommerce.net/)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License,version 3 (GPL-3.0)
 * @author Mobicommerce <info@mobicommerce.net>
 */

namespace Mobicommerce\Mobiapp\Model;

use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Exception\NotFoundException;
use Magento\Framework\Event\ManagerInterface as EventManager;
use Mobicommerce\Mobiapp\Helper\Image as ImageHelper;
use Mobicommerce\Mobiapp\Helper\DbHelper;

class SectionRepository
{
    public const TBL_PAGEBUILDER_MAPPING = 'mobicommerce_page_builder_section_mapping';
    public const TBL_WIDGET_MAPPING = 'mobicommerce_section_widget_mapping';

    /**
     * SectionRepository constructor.
     *
     * @param \Mobicommerce\Mobiapp\Model\ResourceModel\Section $resource
     * @param \Mobicommerce\Mobiapp\Model\SectionFactory $sectionFactory
     * @param Magento\Framework\App\ResourceConnection $resourceConnection
     * @param Magento\Framework\Event\ManagerInterface $eventManager
     * @param Mobicommerce\Mobiapp\Helper\Image $imageHelper
     */
    public function __construct(
        \Mobicommerce\Mobiapp\Model\ResourceModel\Section $resource,
        \Mobicommerce\Mobiapp\Model\SectionFactory $sectionFactory,
        ResourceConnection $resourceConnection,
        EventManager $eventManager,
        ImageHelper $imageHelper
    ) {
        $this->resource = $resource;
        $this->sectionFactory = $sectionFactory;
        $this->resourceConnection = $resourceConnection;
        $this->eventManager = $eventManager;
        $this->imageHelper = $imageHelper;
    }

    /**
     * Save
     *
     * @param object $section
     */
    public function save($section)
    {
        $this->resource->save($section);
        $this->saveWidgets($section);
    }

    /**
     * SaveWidgets
     *
     * @param object $section
     */
    public function saveWidgets($section)
    {
        $connection = $this->resourceConnection->getConnection();
        $table = $this->resourceConnection->getTableName(self::TBL_WIDGET_MAPPING);

        $dynamicRowsField = 'widgets_dynamic_rows';
        if ($widgets = $section->getData($dynamicRowsField)) {
            $widgets = $widgets[$dynamicRowsField] ?? false;

            if ($widgets) {
                $allWidgetIds = [];
                foreach ($widgets as $row) {
                    $widgetId = $row['id'];
                    $position = $row['position'];

                    $allWidgetIds[] = $widgetId;

                    $select = $connection->select()
                        ->from(
                            ['mapping' => $table]
                        )->where('section_id = ?', $section->getId())
                        ->where('widget_id = ?', $widgetId);

                    if ($record = $connection->fetchRow($select)) {
                        if ($record['position'] != $position) {
                            $connection->update($table, [
                                'position' => $position
                            ], [
                                'section_id = ?' => $section->getId(),
                                'widget_id = ?' => $widgetId
                            ]);
                        }
                        
                    } else {
                        $connection->insert($table, [
                            'section_id' => $section->getId(),
                            'widget_id' => $widgetId,
                            'position' => $position
                        ]);
                    }
                }
                
                $connection->delete($table, [
                    'section_id = ?' => $section->getId(),
                    'widget_id NOT IN (?)' => $allWidgetIds
                ]);
            }

            $section->unsetData($dynamicRowsField);
        } else {
            $connection->delete($table, [
                "section_id = ?" => $section->getId()
            ]);
        }
    }

    /**
     * Get sections by pagebuilder id
     *
     * @param object $pageBuilderId
     */
    public function getSectionsByPageBuilderId($pageBuilderId)
    {
        $collection = $this->sectionFactory->create()->getCollection();
        $collection->addEnableStatusFilter();

        $mappingTable = $this->resourceConnection->getTableName(self::TBL_PAGEBUILDER_MAPPING);

        $collection->join(
            ['mapping' => $mappingTable],
            'mapping.section_id = main_table.id',
            null
        );
        $collection->getSelect()->where('mapping.page_builder_id = ?', $pageBuilderId);
        $collection->getSelect()->order('position', 'ASC');

        return $this->processSectionsCollection($collection);
    }

    /**
     * Get by ids
     *
     * @param array $ids
     */
    public function getByIds($ids)
    {
        $collection = $this->sectionFactory->create()->getCollection();
        $collection->addFieldToFilter('id', ['in' => $ids]);
        $collection->addEnableStatusFilter();
        $collection->getSelect()->order(
            DbHelper::prepareIdWiseSortOrder($ids)
        );
        return $this->processSectionsCollection($collection);
    }

     /**
      * Process sections collection
      *
      * @param object $collection
      * @return object
      */
    public function processSectionsCollection($collection)
    {
        $sections = [];
        if ($collection->getSize()) {
            foreach ($collection as $item) {
                $section = $item->getData();
                
                foreach (['desktop_image', 'mobile_image'] as $imageColumn) {
                    if ($section[$imageColumn]) {
                        $section[$imageColumn] = $this->imageHelper->getFileUrl($section[$imageColumn]);
                    }
                }

                $sections[] = $section;
            }
        }

        return $sections;
    }

    /**
     * Duplicate by id
     *
     * @param int $id
     * @return object
     */
    public function duplicateById($id)
    {
        $model = $this->sectionFactory->create()->load($id);

        if (!$model->getId()) {
            throw new NotFoundException(__('Item not available to duplicate'));
        }

        $model->setId(null)
            ->setName('Copy of '. $model->getName())
            ->unsetData('created_at')
            ->unsetData('updated_at')
            ->save();

        $this->eventManager->dispatch('mobicommerce_section_duplicate_child_process', [
            'id' => $id,
            'section' => $model
        ]);

        return $model;
    }
}
