<?php
/**
 * Mobicommerce
 * Copyright (C) 2021 Mobicommerce <info@mobicommerce.net>
 *
 * @category Mobicommerce
 * @package Mobicommerce_Mobiapp
 * @copyright Copyright (c) 2021 Mobicommerce (http://www.mobicommerce.net/)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License,version 3 (GPL-3.0)
 * @author Mobicommerce <info@mobicommerce.net>
 */

namespace Mobicommerce\Mobiapp\Model;

use Magento\Framework\Serialize\Serializer\Serialize;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Exception\NotFoundException;
use Magento\Framework\Event\ManagerInterface as EventManager;
use Mobicommerce\Mobiapp\Model\ImageFactory;
use Mobicommerce\Mobiapp\Helper\Image as ImageHelper;
use Magento\Catalog\Model\Layer\Resolver;
use Magento\CatalogGraphQl\Model\Resolver\Products\Query\ProductQueryInterface;
use Mobicommerce\ContentsGraphQl\Model\Resolver\Widgets\Inputs;
use Mobicommerce\Mobiapp\Helper\DbHelper;
use Magento\Cms\Model\Template\FilterProvider;

class WidgetRepository
{
    public const TBL_IMAGE_MAPPING = 'mobicommerce_widget_image_mapping';
    public const TBL_CATEGORY_MAPPING = 'mobicommerce_widget_category_mapping';
    public const TBL_LINK_BUILDER = 'mobicommerce_widget_link_builder';
    public const TBL_SECTION_MAPPING = 'mobicommerce_section_widget_mapping';
    public const TBL_PAGEBUILDER_LINKBUILDER_MAPPING = 'mobicommerce_page_builder_link_builder_widget_mapping';

    /**
     * WidgetRepository constructor.
     *
     * @param \Mobicommerce\Mobiapp\Model\ResourceModel\Widget $resource
     * @param \Mobicommerce\Mobiapp\Model\WidgetFactory $widgetFactory
     * @param Mobicommerce\Mobiapp\Model\ImageFactory $imageFactory
     * @param Magento\Framework\Serialize\Serializer\Serialize $serialize
     * @param Magento\Framework\Serialize\Serializer\Json $json
     * @param Magento\Framework\App\ResourceConnection $resourceConnection
     * @param Magento\Framework\Event\ManagerInterface $eventManager
     * @param Mobicommerce\Mobiapp\Helper\Image $imageHelper
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Cms\Model\PageFactory $cmsPageFactory
     * @param \Mobicommerce\Mobiapp\Model\ProductcollectionRepository $productcollectionRepository
     * @param \Mobicommerce\Mobiapp\Ui\DataProvider\Productcollection\Preview\Filter\ConditionsFilter $conditionsFilter
     * @param Magento\CatalogGraphQl\Model\Resolver\Products\Query\ProductQueryInterface $searchQuery
     * @param Mobicommerce\ContentsGraphQl\Model\Resolver\Widgets\Inputs $inputs
     * @param FilterProvider $filterProvider
     */

    public function __construct(
        \Mobicommerce\Mobiapp\Model\ResourceModel\Widget $resource,
        \Mobicommerce\Mobiapp\Model\WidgetFactory $widgetFactory,
        ImageFactory $imageFactory,
        Serialize $serialize,
        Json $json,
        ResourceConnection $resourceConnection,
        EventManager $eventManager,
        ImageHelper $imageHelper,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Cms\Model\PageFactory $cmsPageFactory,
        \Mobicommerce\Mobiapp\Model\ProductcollectionRepository $productcollectionRepository,
        \Mobicommerce\Mobiapp\Ui\DataProvider\Productcollection\Preview\Filter\ConditionsFilter $conditionsFilter,
        ProductQueryInterface $searchQuery,
        Inputs $inputs,
        FilterProvider $filterProvider
    ) {
        $this->resource = $resource;
        $this->widgetFactory = $widgetFactory;
        $this->imageFactory = $imageFactory;
        $this->serialize = $serialize;
        $this->json = $json;
        $this->resourceConnection = $resourceConnection;
        $this->eventManager = $eventManager;
        $this->imageHelper = $imageHelper;
        $this->storeManager = $storeManager;
        $this->cmsPageFactory = $cmsPageFactory;
        $this->productcollectionRepository = $productcollectionRepository;
        $this->conditionsFilter = $conditionsFilter;
        $this->searchQuery = $searchQuery;
        $this->inputs = $inputs;
        $this->filterProvider = $filterProvider;
    }

    /**
     * Save
     *
     * @param object $widget
     */
    public function save($widget)
    {
        $this->setDataForWidgetType($widget);
        $this->resource->save($widget);
        $this->saveBannersForWidgets($widget);
        $this->saveCategoriesForWidgets($widget);
        $this->saveLinkBuilderForWidgets($widget);
    }

    /**
     * Duplicate by Id
     *
     * @param int $id
     */
    public function duplicateById($id)
    {
        $widget = $this->widgetFactory->create()->load($id);

        if (!$widget->getId()) {
            throw new NotFoundException(__('Item not available to duplicate'));
        }

        $widget->setId(null)
            ->setName('Copy of '. $widget->getName())
            ->unsetData('created_at')
            ->unsetData('updated_at')
            ->save();

        $this->eventManager->dispatch('mobicommerce_widget_duplicate_child_process', [
            'id' => $id,
            'widget' => $widget
        ]);

        return $widget;
    }

    /**
     * Save banners for widgets
     *
     * @param object $widget
     */
    public function saveBannersForWidgets($widget)
    {
        $connection = $this->resourceConnection->getConnection();
        $table = $this->resourceConnection->getTableName(self::TBL_IMAGE_MAPPING);

        if ($banners = $widget->getData('banners')) {
            $dynamicRowsField = 'images_dynamic_rows';
            $banners = $banners[$dynamicRowsField][$dynamicRowsField] ?? false;

            if ($banners) {
                $allImageIds = [];
                foreach ($banners as $row) {
                    $imageId = $row['id'];
                    $position = $row['position'];

                    $allImageIds[] = $imageId;

                    $select = $connection->select()
                        ->from(
                            ['mapping' => $table]
                        )->where('widget_id = ?', $widget->getId())
                        ->where('image_id = ?', $imageId);

                    if ($record = $connection->fetchRow($select)) {
                        if ($record['position'] != $position) {
                            $connection->update($table, [
                                'position' => $position
                            ], [
                                'widget_id = ?' => $widget->getId(),
                                'image_id = ?' => $imageId
                            ]);
                        }
                    } else {
                        $connection->insert($table, [
                            'widget_id' => $widget->getId(),
                            'image_id' => $imageId,
                            'position' => $position
                        ]);
                    }
                }
                
                $connection->delete($table, [
                    'widget_id = ?' => $widget->getId(),
                    'image_id NOT IN (?)' => $allImageIds
                ]);
            }

            $widget->unsetData('banners');
        } else {
            $connection->delete($table, [
                "widget_id = ?" => $widget->getId()
            ]);
        }
    }

    /**
     * Save categories for widgets
     *
     * @param object $widget
     */
    public function saveCategoriesForWidgets($widget)
    {
        if ($categories = $widget->getData('categories')) {
            $categoryIds = $categories['category_ids'] ?? null;
            if ($categoryIds) {
                $table = $this->resourceConnection->getTableName(self::TBL_CATEGORY_MAPPING);
                $connection = $this->resourceConnection->getConnection();
                $select = $connection->select()
                    ->from(
                        ['mapping' => $table]
                    )->where('widget_id = ?', $widget->getId())
                    ->reset(\Magento\Framework\DB\Select::COLUMNS)
                    ->columns(['GROUP_CONCAT(category_id) AS db_category_ids']);
                $dbCategoryIds = $connection->fetchOne($select);
                if ($dbCategoryIds) {
                    $dbCategoryIds = explode(',', $dbCategoryIds);
                }

                if (!$dbCategoryIds) {
                    $dbCategoryIds = [];
                }

                $categoryIdsToInsert = array_diff($categoryIds, $dbCategoryIds);
                if ($categoryIdsToInsert) {
                    $bulkInsert = [];
                    foreach ($categoryIdsToInsert as $categoryId) {
                        $bulkInsert[] = [
                            'widget_id' => $widget->getId(),
                            'category_id' => $categoryId
                        ];
                    }

                    $connection->insertMultiple($table, $bulkInsert);
                }

                $categoryIdsToDelete = array_diff($dbCategoryIds, $categoryIds);
                if ($categoryIdsToDelete) {
                    $connection->delete(
                        $table,
                        [
                            'widget_id = ?' => $widget->getId(),
                            'category_id IN (?)' => $categoryIdsToDelete
                        ]
                    );
                }
            }

            $widget->unsetData('categories');
        }

        if ($categoryPosition = $widget->getData('widget_category_position')) {
            $categoryPosition = $this->json->unserialize($categoryPosition);
            if ($categoryPosition) {
                $table = $this->resourceConnection->getTableName(self::TBL_CATEGORY_MAPPING);
                $connection = $this->resourceConnection->getConnection();

                foreach ($categoryPosition as $categoryId => $position) {
                    $connection->update($table, [
                        'position' => $position
                    ], [
                        'widget_id = ?' => $widget->getId(),
                        'category_id = ?' => $categoryId
                    ]);
                }
            }

            $widget->unsetData('widget_category_position');
        }
    }

     /**
      * Save link builder for widgets
      *
      * @param object $widget
      */
    public function saveLinkBuilderForWidgets($widget)
    {
        if ($links = $widget->getData('links')) {
            $linkIds = [];

            $table = $this->resourceConnection->getTableName(self::TBL_LINK_BUILDER);
            $connection = $this->resourceConnection->getConnection();

            $dynamicRows = $links['dynamic_rows_links']['dynamic_rows_links'] ?? [];
            if ($dynamicRows) {
                $bulkInsert = [];
                foreach ($dynamicRows as $item) {
                    $linkId = $item['id'] ?? false;

                    if (!$linkId) {
                        $bulkInsert[] = [
                            'widget_id' => $widget->getId(),
                            'label' => $item['label'],
                            'deeplink' => $item['deeplink'],
                            'position' => $item['position']
                        ];
                    } else {
                        $linkIds[] = $linkId;
                        $connection->update($table, [
                            'label' => $item['label'],
                            'deeplink' => $item['deeplink'],
                            'position' => $item['position']
                        ], [
                            'id = ?' => $linkId,
                            'widget_id = ?' => $widget->getId()
                        ]);
                    }
                }

                $select = $connection->select()
                    ->from(
                        ['links' => $table]
                    )->where('widget_id = ?', $widget->getId())
                    ->reset(\Magento\Framework\DB\Select::COLUMNS)
                    ->columns(['GROUP_CONCAT(id) AS db_link_ids']);

                $dbLinkIds = $connection->fetchOne($select);
                if ($dbLinkIds) {
                    $dbLinkIds = explode(',', $dbLinkIds);
                }

                if (!$dbLinkIds) {
                    $dbLinkIds = [];
                }

                $linkIdsToDelete = array_diff($dbLinkIds, $linkIds);
                if ($linkIdsToDelete) {
                    $connection->delete(
                        $table,
                        [
                            'widget_id = ?' => $widget->getId(),
                            'id IN (?)' => $linkIdsToDelete
                        ]
                    );
                }

                if ($bulkInsert) {
                    $connection->insertMultiple($table, $bulkInsert);
                }
            }

            $widget->unsetData('links');
        }
    }

     /**
      * Set data for widgettype
      *
      * @param object $widget
      */
    public function setDataForWidgetType($widget)
    {
        switch ($widget->getType()) {
            case 'banner_slider':
                $this->setDataForBannerSlider($widget);
                break;

            case 'image_slider':
                $this->setDataForImageSlider($widget);
                break;

            case 'image_grid':
                $this->setDataForImageGrid($widget);
                break;

            case '3_image_container':
                $this->setDataFor3ImageContainer($widget);
                break;

            case '4_image_container':
                $this->setDataFor4ImageContainer($widget);
                break;

            case '5_image_container':
                $this->setDataFor5ImageContainer($widget);
                break;

            case '6_image_container':
                $this->setDataFor6ImageContainer($widget);
                break;

            case '7_image_container':
                $this->setDataFor7ImageContainer($widget);
                break;

            case 'product_carousel':
                $this->setDataForProductCarousel($widget);
                break;

            case 'product_list':
                $this->setDataForProductList($widget);
                break;

            case 'category_carousel':
                $this->setDataForCategoryCarousel($widget);
                break;

            case 'category_list':
                $this->setDataForCategoryList($widget);
                break;

            case 'html':
                $this->setDataForHtml($widget);
                break;

            case 'image_mapping':
                $this->setDataForImageMapping($widget);
                break;

            case 'link_builder':
                $this->setDataForLinkBuilder($widget);
                break;
            
            default:
                # code...
                break;
        }
    }

     /**
      * Set data for banner slider
      *
      * @param object $widget
      */
    public function setDataForBannerSlider($widget)
    {
        if ($general = $widget->getData('general')) {
            foreach ($general as $key => $value) {
                $widget->setData($key, $value);
            }
            
            $widget->setData('widget_specific_options', $this->serialize->serialize([
                    'auto_slide' => $general['auto_slide'],
                    'center_mode' => $general['center_mode'],
                    'auto_slide_time' => $general['auto_slide_time'],
                    'show_arrows' => $general['show_arrows'],
                    'show_pagination' => $general['show_pagination'],
                    'images_effect' => $general['images_effect'],
                ]));
            $widget->unsetData('general');
        }
    }

     /**
      * Set data for image slider
      *
      * @param object $widget
      */
    public function setDataForImageSlider($widget)
    {
        if ($general = $widget->getData('general')) {
            foreach ($general as $key => $value) {
                $widget->setData($key, $value);
            }

            $widget->setData('widget_specific_options', $this->serialize->serialize([
                    'auto_slide' => $general['auto_slide'],
                    'auto_slide_time' => $general['auto_slide_time'],
                    'show_arrows' => $general['show_arrows'],
                    'show_pagination' => $general['show_pagination'],
                    'column_per_view' => $general['column_per_view'],
                    'slides_per_column' => $general['slides_per_column'],
                    'desktop_column_per_view' => $general['desktop_column_per_view'],
                    'images_effect' => $general['images_effect'],
                    'show_in_flat_design' => $general['show_in_flat_design'],
                    'desktop_slides_per_column' => $general['desktop_slides_per_column'],
                ]));
            $widget->unsetData('general');
        }
    }

     /**
      * Set data for image grid
      *
      * @param object $widget
      */
    public function setDataForImageGrid($widget)
    {
        if ($general = $widget->getData('general')) {
            foreach ($general as $key => $value) {
                $widget->setData($key, $value);
            }
            
            $widget->setData('widget_specific_options', $this->serialize->serialize([
                    'images_per_row' => $general['images_per_row'],
                    'desktop_images_per_row' => $general['desktop_images_per_row'],
                    'images_effect' => $general['images_effect'],
                    'show_in_flat_design' => $general['show_in_flat_design']
                ]));
            $widget->unsetData('general');
        }
    }

     /**
      * Set data for3image container
      *
      * @param object $widget
      */
    public function setDataFor3ImageContainer($widget)
    {
        if ($general = $widget->getData('general')) {
            foreach ($general as $key => $value) {
                $widget->setData($key, $value);
            }
            
            $widget->setData('widget_specific_options', $this->serialize->serialize([
                    'show_right_to_left' => $general['show_right_to_left'],
                    'show_in_flat_design' => $general['show_in_flat_design'],
                    'responsibleness' => $general['responsibleness'],
                    'images_effect' => $general['images_effect']
                ]));
            $widget->unsetData('general');
        }
    }

      /**
       * Set data for4image container
       *
       * @param object $widget
       */
    public function setDataFor4ImageContainer($widget)
    {
        if ($general = $widget->getData('general')) {
            foreach ($general as $key => $value) {
                $widget->setData($key, $value);
            }
            
            $widget->setData('widget_specific_options', $this->serialize->serialize([
                    'show_in_flat_design' => $general['show_in_flat_design'],
                    'responsibleness' => $general['responsibleness'],
                    'images_effect' => $general['images_effect']
                ]));
            $widget->unsetData('general');
        }
    }

    /**
     * Set data for5image container
     *
     * @param object $widget
     */
    public function setDataFor5ImageContainer($widget)
    {
        if ($general = $widget->getData('general')) {
            foreach ($general as $key => $value) {
                $widget->setData($key, $value);
            }
           // echo "<pre>"; print_r($widget->getData('general')); exit;
            $widget->setData('widget_specific_options', $this->serialize->serialize([
                    'show_right_to_left' => $general['show_right_to_left'],
                    'show_in_flat_design' => $general['show_in_flat_design'],
                    'responsibleness' => $general['responsibleness'],
                    'images_effect' => $general['images_effect']
                ]));
            $widget->unsetData('general');
        }
    }

     /**
      * Set data for6image container
      *
      * @param object $widget
      */
    public function setDataFor6ImageContainer($widget)
    {
        if ($general = $widget->getData('general')) {
            foreach ($general as $key => $value) {
                $widget->setData($key, $value);
            }
            
            $widget->setData('widget_specific_options', $this->serialize->serialize([
                    'show_in_flat_design' => $general['show_in_flat_design'],
                    'responsibleness' => $general['responsibleness'],
                    'images_effect' => $general['images_effect']
                ]));
            $widget->unsetData('general');
        }
    }

     /**
      * Set data for7image container
      *
      * @param object $widget
      */
    public function setDataFor7ImageContainer($widget)
    {
        if ($general = $widget->getData('general')) {
            foreach ($general as $key => $value) {
                $widget->setData($key, $value);
            }
            
            $widget->setData('widget_specific_options', $this->serialize->serialize([
                    'show_right_to_left' => $general['show_right_to_left'],
                    'show_in_flat_design' => $general['show_in_flat_design'],
                    'responsibleness' => $general['responsibleness'],
                    'images_effect' => $general['images_effect']
                ]));
            $widget->unsetData('general');
        }
    }
    
     /**
      * Set data for product carousel
      *
      * @param object $widget
      */
    public function setDataForProductCarousel($widget)
    {
        if ($general = $widget->getData('general')) {
            foreach ($general as $key => $value) {
                $widget->setData($key, $value);
            }

            $widget->setData('widget_specific_options', $this->serialize->serialize([
                    'show_name' => $general['show_name'],
                    'show_price' => $general['show_price'],
                    'show_in_flat_design' => $general['show_in_flat_design'],
                    'show_view_all_link' => $general['show_view_all_link'],
                    'max_products_to_show_on_widget' => $general['max_products_to_show_on_widget']
                ]));
            $widget->unsetData('general');
        }
       
        if ($mobicommerceCmsPage = $widget->getData('mobicommerce_cms_page')) {
            $widget->setData('mobicommerce_cms_page_id', $mobicommerceCmsPage['mobicommerce_cms_page_id']);
            $widget->unsetData('mobicommerce_cms_page');
        }
    }

     /**
      * Set data for productlist
      *
      * @param object $widget
      */
    public function setDataForProductList($widget)
    {
        if ($general = $widget->getData('general')) {
            foreach ($general as $key => $value) {
                $widget->setData($key, $value);
            }
            
            $widget->setData('widget_specific_options', $this->serialize->serialize([
                    'show_name' => $general['show_name'],
                    'show_price' => $general['show_price'],
                    'show_in_flat_design' => $general['show_in_flat_design'],
                    'show_view_all_link' => $general['show_view_all_link'],
                    'max_products_to_show_on_widget' => $general['max_products_to_show_on_widget']
                ]));
            $widget->unsetData('general');
        }

        if ($mobicommerceCmsPage = $widget->getData('mobicommerce_cms_page')) {
            $widget->setData('mobicommerce_cms_page_id', $mobicommerceCmsPage['mobicommerce_cms_page_id']);
            $widget->unsetData('mobicommerce_cms_page');
        }
    }

     /**
      * Set data for category carousel
      *
      * @param object $widget
      */
    public function setDataForCategoryCarousel($widget)
    {
        if ($general = $widget->getData('general')) {
            foreach ($general as $key => $value) {
                $widget->setData($key, $value);
            }
            
            $widget->setData('widget_specific_options', $this->serialize->serialize([
                    'show_name' => $general['show_name'],
                    'show_thumbnail' => $general['show_thumbnail'],
                    'force_navigate_to_product_list' => $general['force_navigate_to_product_list']
                ]));
            $widget->unsetData('general');
        }
    }

     /**
      * Set data for cateogry list
      *
      * @param object $widget
      */
    public function setDataForCategoryList($widget)
    {
        if ($general = $widget->getData('general')) {
            foreach ($general as $key => $value) {
                $widget->setData($key, $value);
            }
            
            $widget->setData('widget_specific_options', $this->serialize->serialize([
                    'show_name' => $general['show_name'],
                    'show_thumbnail' => $general['show_thumbnail'],
                    'force_navigate_to_product_list' => $general['force_navigate_to_product_list']
                ]));
            $widget->unsetData('general');
        }
    }
    
     /**
      * Set data for html
      *
      * @param object $widget
      */
    public function setDataForHtml($widget)
    {
        $widget->setData('widget_specific_options', $this->serialize->serialize([
                'content' => $widget->getData('content')
            ]));
    }

     /**
      * Set data for image mapping
      *
      * @param object $widget
      */
    public function setDataForImageMapping($widget)
    {

       // echo "<pre>setDataForImageMapping - 595";
       // print_r($widget->getData('general'));
        $finalImage = "";

        if ($general = $widget->getData('general')) {
            foreach ($general as $key => $value) {
                $widget->setData($key, $value);
            }
            $widget->unsetData('general');
        }

        if ($imageMapping = $widget->getData('image_mapping')) {
            $image = $imageMapping['image'][0] ?? false;
            if (is_array($image)) {
                if (!empty($image['delete'])) {
                    $image = null;
                } else {
                    if (isset($image['name'])) {
                        $image = $image['url'];
                        $image = $this->imageHelper->cleanPathBeforeSave($image);
                        $finalImage = $image;
                    }
                }
            }

            $imageMap = $imageMapping['imagemap'] ?? false;
            $widget->setData('widget_specific_options', $this->serialize->serialize([
                'image' => $finalImage,
                'imagemap' => $imageMap,
                'images_effect' => $general['images_effect'],
                'show_in_flat_design' => $general['show_in_flat_design']
            ]));

            $widget->unsetData('image_mapping');
        }
    }

     /**
      * Set data for link builder
      *
      * @param object $widget
      */
    public function setDataForLinkBuilder($widget)
    {
        if ($general = $widget->getData('general')) {
            foreach ($general as $key => $value) {
                $widget->setData($key, $value);
            }

            $widget->unsetData('general');
        }
    }

     /**
      * Get link builder widgets by page builder id
      *
      * @param object $pageBuilderId
      * @return object
      */
    public function getLinkBuilderWidgetsByPageBuilderId($pageBuilderId)
    {
        $collection = $this->widgetFactory->create()->getCollection();

        $mappingTable = $this->resourceConnection->getTableName(self::TBL_PAGEBUILDER_LINKBUILDER_MAPPING);

        $collection->join(
            ['mapping' => $mappingTable],
            'mapping.link_builder_widget_id = main_table.id',
            null
        );
        $collection->getSelect()->where('mapping.page_builder_id = ?', $pageBuilderId);
        $collection->getSelect()->order('position', 'ASC');

        return $this->processWidgetsCollection($collection);
    }

    /**
     * Get widgetsby section id
     *
     * @param object $sectionId
     * @return object
     */
    public function getWidgetsBySectionId($sectionId)
    {
        $collection = $this->widgetFactory->create()->getCollection();
        $collection->addEnableStatusFilter();
        $mappingTable = $this->resourceConnection->getTableName(self::TBL_SECTION_MAPPING);

        $collection->join(
            ['mapping' => $mappingTable],
            'mapping.widget_id = main_table.id',
            null
        );
        $collection->getSelect()->where('mapping.section_id = ?', $sectionId);
        $collection->getSelect()->order('position', 'ASC');
        
        return $this->processWidgetsCollection($collection);
    }

     /**
      * Attach widget specific option
      *
      * @param object $widget
      * @return object
      */
    private function attachWidgetSpecificOptions(&$widget)
    {
        $widgetSpecificOptions = $widget['widget_specific_options'];
        $widget['widget_specific_options'] = [];
        if ($widgetSpecificOptions) {
            $widgetSpecificOptions = $this->serialize->unserialize($widgetSpecificOptions);

            if ($widgetSpecificOptions) {
                foreach ($widgetSpecificOptions as $code => $value) {
                    if ($code == 'content' && $value) {
                        $value = $this->filterProvider->getPageFilter()->filter($value);
                    }

                    $widget['widget_specific_options'][] = [
                        'code' => $code,
                        'value' => $value
                    ];
                }
            }
        }
    }

    /**
     * Attach banners if applicable
     *
     * @param object $widget
     * @return object
     */
    private function attachBannersIfApplicable(&$widget)
    {
        switch ($widget['type']) {
            case 'banner_slider':
            case 'image_slider':
            case 'image_grid':
            case '3_image_container':
            case '4_image_container':
            case '5_image_container':
            case '6_image_container':
            case '7_image_container':
                $collection = $this->imageFactory->create()->getCollection();

                $mappingTable = $this->resourceConnection->getTableName(self::TBL_IMAGE_MAPPING);
                $collection->join(
                    ['mapping' => $mappingTable],
                    'mapping.image_id = main_table.id',
                    null
                );
                $collection->getSelect()->where('mapping.widget_id = ?', $widget['id']);
                $collection->getSelect()->order('position', 'ASC');

                if ($collection->getSize() == 0) {
                    return;
                }

                $widget['banners'] = [];
                foreach ($collection as $item) {
                    $banner = $item->getData();
                    foreach (['desktop_image', 'mobile_image'] as $imageColumn) {
                        if ($banner[$imageColumn]) {
                            $file = $this->imageHelper->getAbsolutePath($banner[$imageColumn]);
                            // phpcs:ignore Magento2.Functions.DiscouragedFunction
                            list($width, $height) = getimagesize($file);
                            $ratio = round($height / $width, 2);
                            $banner[$imageColumn.'_ratio'] = "1:$ratio";
                            $banner[$imageColumn] = $this->imageHelper->getFileUrl($banner[$imageColumn]);
                        }
                    }

                    $widget['banners'][] = $banner;
                }
                break;
            
            default:
                // code...
                break;
        }
    }

    /**
     * Attach categories if applicable
     *
     * @param object $widget
     * @return object
     */
    private function attachCategoriesIfApplicable(&$widget)
    {
        switch ($widget['type']) {
            case 'category_carousel':
            case 'category_list':
                $categoryIds = [];
                $connection = $this->resourceConnection->getConnection();
                $table = $this->resourceConnection->getTableName(self::TBL_CATEGORY_MAPPING);
                $select = $connection->select()
                    ->from(
                        ['mapping' => $table]
                    )->where('widget_id = ?', $widget['id'])
                    ->order('position', 'ASC')
                    ->reset(\Magento\Framework\DB\Select::COLUMNS)
                    ->columns(['category_id']);
                $collection = $connection->fetchAll($select);
                if ($collection) {
                    foreach ($collection as $item) {
                        $categoryIds[] = (int) $item['category_id'];
                    }

                    $widget['category_ids'] = $categoryIds;
                }
                
                break;
            
            default:
                // code...
                break;
        }
    }

    /**
     * Attach link builders if applicable
     *
     * @param object $widget
     * @return object
     */
    private function attachLinkBuildersIfApplicable(&$widget)
    {
        switch ($widget['type']) {
            case 'link_builder':
                $connection = $this->resourceConnection->getConnection();
                $table = $this->resourceConnection->getTableName(self::TBL_LINK_BUILDER);
                $select = $connection->select()
                    ->from(
                        ['mapping' => $table]
                    )->where('widget_id = ?', $widget['id'])
                    ->order('position', 'ASC')
                    ->reset(\Magento\Framework\DB\Select::COLUMNS)
                    ->columns(['label', 'deeplink']);
                $collection = $connection->fetchAll($select);
                if ($collection) {
                    $widget['links'] = [];
                    foreach ($collection as $item) {
                        $widget['links'][] = $item;
                    }
                }
                break;
            
            default:
                // code...
                break;
        }
    }

    /**
     * Attach cms page products if applicable
     *
     * @param object $widget
     * @return object
     */
    private function attachCmsPageProductsIfApplicable(&$widget)
    {
        switch ($widget['type']) {
            case 'product_carousel':
            case 'product_list':
                $cmsPageId = (int) $widget['mobicommerce_cms_page_id'];
                $cmsPage = $this->cmsPageFactory->create()->load($cmsPageId);
                $data['id'] = $cmsPage->getId();
                $data['url'] = $cmsPage->getIdentifier();

                $productCollectionId = $cmsPage->getMobicommerceProductCollectionId();
                $data['product_collection_id'] = $productCollectionId;
                
                $args = [
                    'pageSize' => 20,
                    'currentPage' => 1,
                    'filter' => [
                        'sku' => [
                            'in' => $this->productcollectionRepository->getMatchedProductSkusByProductCollectionId(
                                $productCollectionId
                            )
                        ]
                    ]
                ];
                if ($widget['widget_specific_options']) {
                    foreach ($widget['widget_specific_options'] as $item) {
                        if ($item['code'] == 'max_products_to_show_on_widget') {
                            $args['pageSize'] = (int) $item['value'];
                        }
                    }
                }
                $info = $this->inputs->getInfo();
                $context = $this->inputs->getContext();

                $searchResult = $this->searchQuery->getResult($args, $info, $context);
                $productResult = [
                    'total_count' => $searchResult->getTotalCount(),
                    'items' => $searchResult->getProductsSearchResult(),
                    'page_info' => [
                        'page_size' => $searchResult->getPageSize(),
                        'current_page' => $searchResult->getCurrentPage(),
                        'total_pages' => $searchResult->getTotalPages()
                    ],
                    'search_result' => $searchResult,
                    'layer_type' => Resolver::CATALOG_LAYER_SEARCH
                ];

                $data['products'] = $productResult;

                $widget['product_cms_page'] = $data;

                break;
            
            default:
                // code...
                break;
        }
    }

    /**
     * Get by ids
     *
     * @param object $ids
     * @return object
     */
    public function getByIds($ids)
    {
        $collection = $this->widgetFactory->create()->getCollection();
        $collection->addFieldToFilter('id', ['in' => $ids]);
        $collection->addEnableStatusFilter();
        $collection->getSelect()->order(
            DbHelper::prepareIdWiseSortOrder($ids)
        );
        return $this->processWidgetsCollection($collection);
    }

    /**
     * Process widgets collection
     *
     * @param object $collection
     * @return object
     */
    public function processWidgetsCollection($collection)
    {
        $widgets = [];
        if ($collection->getSize()) {
            foreach ($collection as $item) {
                $widget = $item->getData();

                $this->attachWidgetSpecificOptions($widget);
                $this->attachBannersIfApplicable($widget);
                $this->attachCategoriesIfApplicable($widget);
                $this->attachLinkBuildersIfApplicable($widget);
                $this->attachCmsPageProductsIfApplicable($widget);

                $widgets[] = $widget;
            }
        }

        return $widgets;
    }
}
