<?php
/**
 * Mobicommerce
 *
 * Copyright (C) 2021 Mobicommerce <info@mobicommerce.net>
 *
 * @category Mobicommerce
 * @package Mobicommerce_Mobiapp
 * @copyright Copyright (c) 2021 Mobicommerce (http://www.mobicommerce.net/)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License,version 3 (GPL-3.0)
 * @author Mobicommerce <info@mobicommerce.net>
 */

namespace Mobicommerce\Mobiapp\Ui\DataProvider\Widget\Form\Modifier;

use Magento\Ui\DataProvider\Modifier\ModifierInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\UrlInterface;
use Magento\Framework\Serialize\Serializer\Serialize;
use Magento\Framework\App\ResourceConnection;
use Mobicommerce\Mobiapp\Ui\DataProvider\Deeplink\Form\ModalMetaProvider as DeeplinkModalMetaProvider;
use Mobicommerce\Mobiapp\Helper\Image as ImageHelper;
use Magento\Ui\Component\DynamicRows;
use Magento\Ui\Component\Form\Element\DataType\Number;
use Magento\Ui\Component\Form\Element\DataType\Text;
use Magento\Ui\Component\Form\Element\Input;
use Magento\Ui\Component\Form\Element\Hidden;
use Magento\Ui\Component\Form\Element\Select;
use Magento\Ui\Component\Form\Field;
use Magento\Ui\Component\Form\Fieldset;
use Magento\Framework\Phrase;

abstract class AbstractModifier implements ModifierInterface
{
    public const TBL_WIDGET_IMAGE_MAPPING = 'mobicommerce_widget_image_mapping';

    /**
     * @var Magento\Framework\App\RequestInterface
     */
    protected $request;

    /**
     * @var Magento\Framework\UrlInterface
     */
    protected $urlBuilder;
    
    /**
     * AbstractModifier constructor.
     *
     * @param RequestInterface $request
     * @param UrlInterface $urlBuilder
     * @param Serialize $serialize
     * @param ResourceConnection $resourceConnection
     * @param DeeplinkModalMetaProvider $deeplinkModalMetaProvider
     * @param arrImageHelperay $imageHelper
     */
    public function __construct(
        RequestInterface $request,
        UrlInterface $urlBuilder,
        Serialize $serialize,
        ResourceConnection $resourceConnection,
        DeeplinkModalMetaProvider $deeplinkModalMetaProvider,
        ImageHelper $imageHelper
    ) {
        $this->request = $request;
        $this->urlBuilder = $urlBuilder;
        $this->serialize = $serialize;
        $this->resourceConnection = $resourceConnection;
        $this->deeplinkModalMetaProvider = $deeplinkModalMetaProvider;
        $this->imageHelper = $imageHelper;
    }

    /**
     * BindGenericDefaultData data
     *
     * @param array $data
     */
    public function bindGenericDefaultData(&$data)
    {
        $id = $this->request->getParam('id');

        $masterData = $data['master_data'];
        if ($masterData['type']) {
            $widgetSpecificOptions = $masterData['widget_specific_options'];
            $data[$id]['general'] = $masterData;

            if ($widgetSpecificOptions) {
                $widgetSpecificOptions = $this->serialize->unserialize($widgetSpecificOptions);
                if ($widgetSpecificOptions) {
                    foreach ($widgetSpecificOptions as $key => $value) {
                        $data[$id]['general'][$key] = $value;
                    }
                }
            }
        }
    }

    /**
     * BindGenericTablessDefaultData data
     *
     * @param array $data
     */
    public function bindGenericTablessDefaultData(&$data)
    {
        $id = $this->request->getParam('id');

        $masterData = $data['master_data'];
        if ($masterData['type']) {
            $widgetSpecificOptions = $masterData['widget_specific_options'];
            $widgetSpecificOptions = $this->serialize->unserialize($widgetSpecificOptions);

            $data[$id] = $masterData;

            foreach ($widgetSpecificOptions as $key => $value) {
                $data[$id][$key] = $value;
            }
        }
    }

    /**
     * BindDbCategories data
     *
     * @param array $data
     */
    public function bindDbCategories(&$data)
    {
        $id = $this->request->getParam('id');

        if ($id) {
            $connection = $this->resourceConnection->getConnection();
            $table = $this->resourceConnection->getTableName('mobicommerce_widget_category_mapping');
            $select = $connection->select()
                ->from(
                    ['mapping' => $table]
                )->where('widget_id = ?', $id)
                ->reset(\Magento\Framework\DB\Select::COLUMNS)
                ->columns(['GROUP_CONCAT(category_id) AS db_category_ids']);
            $dbCategoryIds = $connection->fetchOne($select);
            if ($dbCategoryIds) {
                $data[$id]['categories']['category_ids'] = explode(',', $dbCategoryIds);
            }
        }
    }

    /**
     * BindBannersData data
     *
     * @param array $data
     */
    public function bindBannersData(&$data)
    {
        $id = $this->request->getParam('id');

        if ($id) {
            $connection = $this->resourceConnection->getConnection();
            $table = $this->resourceConnection->getTableName(self::TBL_WIDGET_IMAGE_MAPPING);

            $select = $connection->select()
                ->from(
                    ['mapping' => $table]
                )->join(
                    ['mobicommerce_image' => $this->resourceConnection->getTableName('mobicommerce_image')],
                    'mobicommerce_image.id=mapping.image_id'
                )->where('widget_id = ?', $id)
                ->order('position ASC');

            $result = $connection->fetchAll($select);

            if ($result) {
                $sortedResult = [];
                foreach ($result as $row) {
                    $row['record_id'] = $row['position'];
                    $row['initialize'] = true;
                    $sortedResult[] = $row;
                }

                $dynamicRowsField = 'images_dynamic_rows';
                $data[$id]['banners'][$dynamicRowsField][$dynamicRowsField] = $sortedResult;
            }
        }
    }

    /**
     * ModifyBannersMeta data
     *
     * @param array $meta
     * @param array $formName
     */
    public function modifyBannersMeta($meta, $formName)
    {
        $meta['banners'] = [
            'arguments' => [
                'data' => [
                    'config' => [
                        'componentType' => 'fieldset',
                        'label' => __('Banners')
                    ]
                ]
            ]
        ];
        
        $meta['banners']['children']['button_set'] = [
            'arguments' => [
                'data' => [
                    'config' => [
                        'formElement' => 'container',
                        'componentType' => 'container',
                        'label' => __('Banners'),
                        'content' => __('Assign Images to this Widget'),
                        'template' => 'ui/form/components/complex',
                        'sortOrder' => 30
                    ]
                ]
            ],
            'children' => [
                'button_widget' => [
                    'arguments' => [
                        'data' => [
                            'config' => [
                                'formElement' => 'container',
                                'componentType' => 'container',
                                'component' => 'Magento_Ui/js/form/components/button',
                                'actions' => [
                                    [
                                        'targetName' => 'index = image_listing_modal',
                                        'actionName' => 'openModal'
                                    ],
                                    [
                                        'targetName' => 'index = mobicommerce_widget_image_listing',
                                        'actionName' => 'render'
                                    ]
                                ],
                                'title' => __('Add Images'),
                                'provider' => null
                            ]
                        ]
                    ]
                ]
            ]
        ];

        $meta['banners']['children']['image_listing_modal'] = [
            'arguments' => [
                'data' => [
                    'config' => [
                        'componentType' => 'modal',
                        'dataScope' => '',
                        'options' => [
                            'title' => __('Add Images'),
                            'buttons' => [
                                [
                                    'text' => __('Cancel'),
                                    'actions' => ['closeModal']
                                ],
                                [
                                    'text' => __('Add Selected Images'),
                                    'class' => 'action-primary',
                                    'actions' => [
                                        [
                                            'targetName' => 'index = mobicommerce_widget_image_listing',
                                            'actionName' => 'save',
                                        ],
                                        'closeModal'
                                    ]
                                ]
                            ]
                        ]
                    ]
                ]
            ],
            'children' => [
                'mobicommerce_widget_image_listing' => [
                    'arguments' => [
                        'data' => [
                            'config' => [
                                'autoRender' => false,
                                'componentType' => 'insertListing',
                                'dataScope' => 'mobicommerce_widget_image_listing',
                                'externalProvider' => implode('.', [
                                    'mobicommerce_widget_image_listing',
                                    'mobicommerce_widget_image_listing_data_source'
                                ]),
                                'selectionsProvider' => implode('.', [
                                    'mobicommerce_widget_image_listing',
                                    'mobicommerce_widget_image_listing',
                                    'image_columns',
                                    'ids'
                                ]),
                                'ns' => 'mobicommerce_widget_image_listing',
                                'render_url' => $this->urlBuilder->getUrl('mui/index/render'),
                                'realTimeLink' => true,
                                'dataLinks' => [
                                    'imports' => false,
                                    'exports' => true
                                ],
                                'behaviourType' => 'simple',
                                'externalFilterMode' => true
                            ]
                        ]
                    ]
                ]
            ]
        ];

        $meta['banners']['children']['images_dynamic_rows'] = [
            'arguments' => [
                'data' => [
                    'config' => [
                        'additionalClasses' => 'admin__field-wide',
                        'componentType' => DynamicRows::NAME,
                        'label' => null,
                        'columnsHeader' => false,
                        'columnsHeaderAfterRender' => true,
                        'renderDefaultRecord' => false,
                        'template' => 'ui/dynamic-rows/templates/grid',
                        'component' => 'Magento_Ui/js/dynamic-rows/dynamic-rows-grid',
                        'addButton' => false,
                        'recordTemplate' => 'record',
                        'dataScope' => 'images_dynamic_rows',
                        'deleteButtonLabel' => __('Remove'),
                        'dataProvider' => 'data.banners.mobicommerce_widget_image_listing',
                        'map' => [
                            'id' => 'id',
                            'name' => 'name',
                            'mobile_image' => 'mobile_image',
                            'desktop_image' => 'desktop_image',
                            'deeplink' => 'deeplink',
                        ],
                        'links' => [
                            'insertData' => '${ $.provider }:${ $.dataProvider }',
                            '__disableTmpl' => ['insertData' => false],
                        ],
                        'sortOrder' => 40,
                    ]
                ]
            ],
            'children' => [
                'record' => [
                    'arguments' => [
                        'data' => [
                            'config' => [
                                'componentType' => 'container',
                                'isTemplate' => true,
                                'is_collection' => true,
                                'component' => 'Magento_Ui/js/dynamic-rows/record',
                                'dataScope' => '',
                            ],
                        ],
                    ],
                    'children' => [
                        'id' => $this->getUrlColumn('id', false, __('ID'), 0),
                        'name' => $this->getTextColumn('name', false, __('Name for Admin'), 20),
                        'mobile_image' => $this->getThumbnailColumn('mobile_image', false, __('Mobile Image'), 30),
                        'desktop_image' => $this->getThumbnailColumn('desktop_image', false, __('Desktop Image'), 40),
                        'deeplink' => $this->getTextColumn('deeplink', false, __('Deeplink'), 50),
                        'actionDelete' => [
                            'arguments' => [
                                'data' => [
                                    'config' => [
                                        'additionalClasses' => 'data-grid-actions-cell',
                                        'componentType' => 'actionDelete',
                                        'dataType' => Text::NAME,
                                        'label' => __('Actions'),
                                        'sortOrder' => 70,
                                        'fit' => true,
                                    ],
                                ],
                            ],
                        ],
                        'position' => [
                            'arguments' => [
                                'data' => [
                                    'config' => [
                                        'columnsHeaderClasses' => 'mobicommerce_hidden_element',
                                        'dataType' => Number::NAME,
                                        'formElement' => Hidden::NAME,
                                        'componentType' => Field::NAME,
                                        'dataScope' => 'position',
                                        'sortOrder' => 80,
                                        'visible' => true,
                                    ],
                                ],
                            ],
                        ],
                    ],
                ]
            ]
        ];

        return $meta;
    }

    /**
     * GetTextColumn data
     *
     * @param String $dataScope
     * @param String $fit
     * @param Phrase $label
     * @param String $sortOrder
     */
    protected function getTextColumn($dataScope, $fit, Phrase $label, $sortOrder)
    {
        $column = [
            'arguments' => [
                'data' => [
                    'config' => [
                        'componentType' => Field::NAME,
                        'formElement' => Input::NAME,
                        'elementTmpl' => 'ui/dynamic-rows/cells/text',
                        'component' => 'Magento_Ui/js/form/element/text',
                        'dataType' => Text::NAME,
                        'dataScope' => $dataScope,
                        'fit' => $fit,
                        'label' => $label,
                        'sortOrder' => $sortOrder,
                    ],
                ],
            ],
        ];

        return $column;
    }

    /**
     * GetUrlColumn data
     *
     * @param String $dataScope
     * @param String $fit
     * @param Phrase $label
     * @param String $sortOrder
     */
    protected function getUrlColumn($dataScope, $fit, Phrase $label, $sortOrder)
    {
        $column = [
            'arguments' => [
                'data' => [
                    'config' => [
                        'componentType' => Field::NAME,
                        'formElement' => Input::NAME,
                        'elementTmpl' => 'Mobicommerce_Mobiapp/dynamic-rows/cells/url',
                        'component' => 'Mobicommerce_Mobiapp/js/dynamic-rows/form/element/url',
                        'dataType' => Text::NAME,
                        'dataScope' => $dataScope,
                        'fit' => $fit,
                        'label' => $label,
                        'sortOrder' => $sortOrder,
                        'href' => $this->urlBuilder->getUrl(
                            'mobicommerce/image/edit',
                            [
                                'id' => '__ID__'
                            ]
                        )
                    ],
                ],
            ],
        ];

        return $column;
    }

    /**
     * GetThumbnailColumn data
     *
     * @param String $dataScope
     * @param String $fit
     * @param Phrase $label
     * @param String $sortOrder
     */
    protected function getThumbnailColumn($dataScope, $fit, Phrase $label, $sortOrder)
    {
        $column = [
            'arguments' => [
                'data' => [
                    'config' => [
                        'componentType' => Field::NAME,
                        'formElement' => Input::NAME,
                        'elementTmpl' => 'ui/dynamic-rows/cells/thumbnail',
                        'component' => 'Magento_Ui/js/form/element/text',
                        'dataType' => Text::NAME,
                        'dataScope' => $dataScope,
                        'fit' => $fit,
                        'label' => $label,
                        'sortOrder' => $sortOrder,
                    ],
                ],
            ],
        ];

        return $column;
    }
}
