<?php

namespace Mobicommerce\UrlRewriteGraphQl\Model\Resolver;

use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Framework\GraphQl\Config\Element\Field;
use Magento\Framework\GraphQl\Exception\GraphQlInputException;
use Magento\Framework\GraphQl\Query\ResolverInterface;
use Magento\Framework\GraphQl\Schema\Type\ResolveInfo;
use Magento\GraphQl\Model\Query\ContextInterface;
use Magento\Framework\HTTP\Client\Curl;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Framework\UrlInterface;
use Magento\UrlRewriteGraphQl\Model\Resolver\EntityUrl;
use Magento\Catalog\Api\CategoryRepositoryInterface;
use Magento\Store\Model\StoreManagerInterface;

class EntityInfo implements ResolverInterface
{
    private const CURL_CONNECTTIMEOUT = 10;
    private const CURL_TIMEOUT = 15;

    /**
     * @var Curl
     */
    private Curl $curl;

    /**
     * @var Json
     */
    private Json $json;

    /**
     * @var UrlInterface
     */
    private UrlInterface $url;

    /**
     * @var EntityUrl
     */
    private EntityUrl $entityUrl;

    /**
     * @var ProductRepositoryInterface
     */
    private ProductRepositoryInterface $productRepository;

    /**
     * @var CategoryRepositoryInterface
     */
    private CategoryRepositoryInterface $categoryRepository;

    /**
     * @var StoreManagerInterface
     */
    private StoreManagerInterface $storeManager;

    /**
     * Constructor
     *
     * @param Curl $curl
     * @param Json $json
     * @param UrlInterface $url
     * @param EntityUrl $entityUrl
     * @param ProductRepositoryInterface $productRepository
     * @param CategoryRepositoryInterface $categoryRepository
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(
        Curl $curl,
        Json $json,
        UrlInterface $url,
        EntityUrl $entityUrl,
        ProductRepositoryInterface $productRepository,
        CategoryRepositoryInterface $categoryRepository,
        StoreManagerInterface $storeManager
    ) {
        $this->curl = $curl;
        $this->json = $json;
        $this->url = $url;
        $this->entityUrl = $entityUrl;
        $this->productRepository = $productRepository;
        $this->categoryRepository = $categoryRepository;
        $this->storeManager = $storeManager;
    }

    /**
     * @inheritdoc
     */
    public function resolve(
        Field $field,
        $context,
        ResolveInfo $info,
        array $value = null,
        array $args = null
    ) {
        $currentStore = (int)$context->getExtensionAttributes()->getStore()->getId();
        $store = $args['destination_store'] ?? false;
        $url = $args['url'] ?? false;
        $entityType = $args['entity_type'] ?? false;
        $entityId = $args['entity_id'] ?? false;

        if (!$store) {
            throw new GraphQlInputException(__('Please provide destination_store'));
        }

        $returnUrl = null;
        if ($url) {
            $entityInfo = $this->entityUrl->resolve(
                $field,
                $context,
                $info,
                $value,
                $args
            );

            $entityType = $entityInfo['type'] ?? false;
            $entityId = $entityInfo['id'] ?? false;
            if ($entityId && in_array($entityType, [
                'CATEGORY',
                'PRODUCT',
                'CMS_PAGE'
            ])) {
                if ($entityType == 'PRODUCT') {
                    try {
                        $product = $this->productRepository->getById((int)$entityId, false, $currentStore);
                        if ($product->getId()) {
                            $entityId = $product->getSku();
                        }
                    } catch (\Exception $e) {

                    }

                    $returnUrl = $this->getUrlByEntityInfo([
                        'store' => $store,
                        'entityType' => strtolower($entityType),
                        'entityId' => $entityId
                    ]);
                } elseif ($entityType == 'CATEGORY') {
                    $category = $this->categoryRepository->get($entityId, $store);
                    $fullUrl = $category->getUrl();
                    $baseUrl = $this->storeManager->getStore($store)->getBaseUrl(UrlInterface::URL_TYPE_WEB);
                    $returnUrl = str_replace(
                        $baseUrl . $store . '/',
                        '',
                        $fullUrl
                    );

                    $baseCurrentUrl = $this->storeManager->getStore($currentStore)->getBaseUrl(UrlInterface::URL_TYPE_WEB);
                    $returnUrl = str_replace(
                        $baseCurrentUrl . $this->storeManager->getStore($currentStore)->getCode() . '/',
                        '',
                        $returnUrl
                    );

                    $returnUrl = str_replace(
                        $baseUrl,
                        '',
                        $returnUrl
                    );

                    $returnUrl = str_replace(
                        $baseCurrentUrl,
                        '',
                        $returnUrl
                    );
                } elseif ($entityType == 'CMS_PAGE') {
                    $returnUrl = $url;
                }
            }else {
                $returnUrl = $url;
            }
        } else {
            if (!$entityType) {
                throw new GraphQlInputException(__('Please provide entity_type'));
            }

            if (!$entityId) {
                throw new GraphQlInputException(__('Please provide entity_id'));
            }

            $returnUrl = $this->getUrlByEntityInfo([
                'store' => $store,
                'entityType' => $entityType,
                'entityId' => $entityId
            ]);
        }

        return [
            'url' => $returnUrl
        ];
    }

    /**
     * Get URL by Entity information
     *
     * @param array $params
     * @return string|null
     */
    private function getUrlByEntityInfo(array $params = []): ?string
    {
        $returnUrl = null;
        $url = $this->url->getCurrentUrl();
        $data = [];

        switch ($params['entityType']) {
            case 'category':
                $data = '{"query":"query {categoryList(filters:{ids:{eq: \\"'.$params['entityId'].'\\"}}){url_key}}"}';
                break;
            case 'product':
                $data = '{"query":"query {products(filter:{sku:{eq:\\"'.$params['entityId'].'\\"}}){items{url_rewrites{url}}}}"}';
                break;
            case 'cms_page':
                $data = '{"query":"query {cmsPage(id: '.((int) $params['entityId']).'){url_key}}"}';
                break;
        }

        $this->curl->addHeader("Store", $params['store']);
        $this->curl->addHeader("Content-Type", "application/json");
        $this->curl->setOption(CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        $this->curl->setOption(CURLOPT_CONNECTTIMEOUT, self::CURL_CONNECTTIMEOUT);
        $this->curl->setOption(CURLOPT_TIMEOUT, self::CURL_TIMEOUT);
        $this->curl->setOption(CURLOPT_RETURNTRANSFER, true);

        try {
            $this->curl->post($url, $data);
            $response = $this->curl->getBody();
            $response = $this->json->unserialize($response);

            switch ($params['entityType']) {
                case 'category':
                    $returnUrl = $response['data']['categoryList'][0]['url_key'] ?? null;
                    break;
                case 'product':
                    $returnUrl = $response['data']['products']['items'][0]['url_rewrites'][0]['url'] ?? null;
                    break;
                case 'cms_page':
                    $returnUrl = $response['data']['cmsPage']['url_key'] ?? null;
                    break;
            }
        } catch (\Exception $e) {
        }

        if (!$returnUrl) {
            $returnUrl = null;
        }

        return $returnUrl;
    }
}
